#!/usr/bin/php
<?php

/* --------------------
Programme 2 und 3 im Rahmen der Lehrveranstaltung
„Kartenverwandte Ausdrucksformen”
unter der Leitung von Karel Kriz an der Universität Wien
-----------------------
Christoph Fink, a0404129
-----------------------

Dieses Skript liest den Inhalt einer ASCII-DEM-Datei 
(ESRI-, nicht USGS-DEM!) in ein zweidimensionales Array und 
errechnet statistische Kennwerte des Datensatzes.

ESRI ArcGrid-Spezifikationen können beispielsweise unter 
http://web.archive.org/web/20070306073851/http://ioc.unesco.org/oceanteacher/OMAP/ArcGrid.htm
gefunden werden.


ABHÄNGIGKEITEN:
 - php-CLI
 - ImageMagick
 - BSD-zip

----------------------*/
define("DEBUG",0);
set_time_limit(0);
ini_set("memory_limit","2048M");


class DEM_Object {
	private $ncols;
	private $nrows;
	private $xllcorner;
	private $yllcorner;
	private $cellsize;
	private $nodata_value=-9999;
	private $daten;
	private $prj="";

	public function __construct() {
		$parameter_anzahl=func_num_args();
		$parameter=func_get_args();

		if(DEBUG){
			echo "DEM_Object-Konstruktor: $parameter_anzahl Parameter übergeben\n";
			print_r($parameter);
		}
	
	//Szenario 1: ein Argument, Dateiname
		if($parameter_anzahl==1 && is_file($parameter[0])){ //gettype erübrigt sich durch is_file

			if(DEBUG) echo "DEM_Object-Konstruktor: erstelle DEM aus Datei ".$parameter[0]."\n";

			$zeilen=file($parameter[0]);

			if(DEBUG) print_r($zeilen);
			if(count($zeilen)<6) exit("DEM_Object-Konstruktor: DEM-Datei ".$parameter[0]." nicht geeignet.\n");

			if(DEBUG) echo "DEM_Object-Konstruktor: nodata_value in zeile5? ".stripos($zeilen[6],"NODATA_VALUE")."\n";

			//diese abfrage ist sehr schlampig ... aber für die aufgabenstellung sollt's reichen ;)
			if(stripos($zeilen[5],"NODATA_VALUE")!==false){
				$header=array_slice($zeilen,0,6);
				$data=array_slice($zeilen,6);
				$nodata_value=(int) $header[5];
			}else{ 
				$header=array_slice($zeilen,0,5);
				$data=array_slice($zeilen,5);
				$nodata_value=-9999;
			}
			unset($zeilen); //speicher frei geben
			

			$zeilenzahl=count($header); //muss hier gesetzt werden, weil größe in der schleife verändert
			for($i=0;$i<$zeilenzahl;$i++){
				if(DEBUG) echo "DEM_Object-Konstruktor: Header Zeile $i: ".$header[$i]." \n";

				$header[$i]=explode(" ",trim($header[$i]));
				$header[$i]=array_filter($header[$i],create_function('$wert','return (trim($wert)!="");'));
				$header[$i]=array_merge($header[$i]); //setzt array-zeiger zurück
				$header[strtolower($header[$i][0])]=$header[$i][1];

				if(DEBUG) print_r($header[$i]);

			}

			if(DEBUG) print_r($header);
			
			$this->ncols=(int) $header["ncols"];
			$this->nrows=(int) $header["nrows"];
			$this->xllcorner=(float) $header["xllcorner"];
			$this->yllcorner=(float) $header["yllcorner"];
			$this->cellsize=(float) $header["cellsize"];

			$zeilennr=0;
			foreach($data as $zeile){
				if(DEBUG) echo "DEM_Object-Konstruktor: bearbeite Datenzeile $zeilennr\n";
				$this->daten[$zeilennr]=explode(" ",trim($zeile));
				$this->daten[$zeilennr]=array_filter($this->daten[$zeilennr],create_function('$wert','return (trim($wert)!="");'));
				array_walk($this->daten[$zeilennr++],create_function('&$wert, $schluessel','$wert=(int) $wert;')); //sonst werden alle werte als string interpretiert -> probleme bei den statistischen funktionen
			}
						
			if(DEBUG) print_r($this->daten);

			if(file_exists(dirname($parameter[0])."/".ShowFileName($parameter[0]).".prj")){
				$this->prj=file_get_contents(dirname($parameter[0])."/".ShowFileName($parameter[0]).".prj");
			}else{
				echo "\nDatei ".ShowFileName($parameter[0]).".prj nicht gefunden, LatLon/WGS84 wird angenommen.\n";
				$this->prj="GEOGCS[\"WGS 84\",\n    DATUM[\"WGS_1984\",\n        SPHEROID[\"WGS 84\",6378137,298.257223563,\n            AUTHORITY[\"EPSG\",\"7030\"]],\n        AUTHORITY[\"EPSG\",\"6326\"]],\n    PRIMEM[\"Greenwich\",0,\n        AUTHORITY[\"EPSG\",\"8901\"]],\n    UNIT[\"degree\",0.01745329251994328,\n        AUTHORITY[\"EPSG\",\"9122\"]],\n    AUTHORITY[\"EPSG\",\"4326\"]]";

			}

			
		}

	//Szenario 2a und 2b: alles extra - 7-8 argumente (5-6 integer/double, 1 array, 1 string)
		else if((($parameter_anzahl==7 && is_array($parameter[5]) && is_string($parameter[6])) || 
			($parameter_anzahl==8 && is_int($parameter[5]) && is_array($parameter[6]) && is_string($parameter[7]))) && 

			is_int($parameter[0]) && is_int($parameter[1]) && 
			(is_int($parameter[2]) || is_float($parameter[2])) &&
			(is_int($parameter[3]) || is_float($parameter[3])) &&
			(is_int($parameter[4]) || is_float($parameter[4]))) {

				if($parameter_anzahl==7){
					if(DEBUG) echo "DEM_Object-Konstruktor: 6 Parameter, erstelle DEM mit default-NODATA_VALUE  (-9999)\n";
					$this->nodata_value=-9999;
					$this->daten=$parameter[5];
				}

				if($parameter_anzahl==8){
					if(DEBUG) echo "DEM_Object-Konstruktor: 7 Parameter, erstelle DEM mit angegebener NODATA_VALUE (".$parameter[5].")\n";
					$this->nodata_value=$parameter[5];
					$this->daten=$parameter[6];
				}
	
				$this->ncols=$parameter[0];
				$this->nrows=$parameter[1];
				$this->xllcorner=$parameter[2];
				$this->yllcorner=$parameter[3];
				$this->cellsize=$parameter[4];
				$this->prj=$parameter[7];

				if(trim($this->prj)==""){
					echo "\nProjektion nicht definiert, latlon/WGS84 angenommen.\n";
					$this->prj="GEOGCS[\"WGS 84\",\n    DATUM[\"WGS_1984\",\n        SPHEROID[\"WGS 84\",6378137,298.257223563,\n            AUTHORITY[\"EPSG\",\"7030\"]],\n        AUTHORITY[\"EPSG\",\"6326\"]],\n    PRIMEM[\"Greenwich\",0,\n        AUTHORITY[\"EPSG\",\"8901\"]],\n    UNIT[\"degree\",0.01745329251994328,\n        AUTHORITY[\"EPSG\",\"9122\"]],\n    AUTHORITY[\"EPSG\",\"4326\"]]";
				}
				
		}
	
	//kein erwartetes Szenario
		else {
			exit("\nDEM_Object-Konstruktor: falsche Parameter übergeben.\n");
		}
		
	}

	public function getValueAt($y,$x){ //Sortierung im Array: zeile,spalte = y,x
		return $this->daten[$x][$y];
	}

	public function getMaximum(){
/*		$maximum=max($this->daten);

		if(DEBUG) print_r($maximum);

		while(is_array($maximum)){
			$maximum=max($maximum);

			if(DEBUG)print_r($maximum);
		}		*/ // elegant aber funktioniert ned ... :/

		$maximum=0;
		for($i=0;$i<$this->getNumberOfRows();$i++){
			if (max($this->daten[$i])>$maximum)
				$maximum=max($this->daten[$i]);
		}
		return $maximum;
	}

	public function getMinimum(){
/*		$minimum=min($this->daten);
		while(is_array($minimum)){
			$minimum=min($minimum);
		}		*/ // elegant aber funktioniert ned ... :/	

		$minimum=min($this->daten[0]);
		for($i=0;$i<$this->getNumberOfRows();$i++){
			if (min($this->daten[$i])<$minimum)
				$minimum=min($this->daten[$i]);
		}

		return $minimum;
	}

	public function getRange(){
		return ($this->getMaximum() - $this->getMinimum());
	}

	public function getMean(){
		return round(array_sum_recursive($this->daten)/(count($this->daten, COUNT_RECURSIVE)-count($this->daten)),2);
	}

	public function getMedian(){
		$hilfsarray=array_flatten($this->daten);

		if(DEBUG) print_r($hilfsarray);
		
		sort($hilfsarray);
		if(count($hilfsarray)%2){
			return $hilfsarray[floor(count($hilfsarray)/2)];
		}else{
		
			return (($hilfsarray[(count($hilfsarray)/2)]+$hilfsarray[(count($hilfsarray)/2-1)])/2);
		}
	}
	
	public function getNumberOfRows(){
		return count($this->daten);
	}

	public function getNumberOfColumns(){
		return (count($this->daten, COUNT_RECURSIVE)-count($this->daten))/count($this->daten);
	}
	
	public function getDEMAs8BitArray() {
		echo "\nBerechne auf 8bit gestauchtes/gestrecktes DEM";
				
		$DEM8bit=$this->daten;
		$faktor=$this->getRange()/255;
		$minimum=(double) $this->getMinimum(); 

		if(DEBUG) echo "getDEMAs8BitArray: Faktor: $faktor Minimum: $minimum\n".'$wert=round(($wert-'.$minimum.')/'.$faktor.',4)'."\n";

		for($i=0;$i<count($DEM8bit);$i++)
			array_walk($DEM8bit[$i],create_function('&$wert, $schluessel','$wert=round(($wert-'.$minimum.')/'.$faktor.',0);'));

		if(DEBUG) print_r($DEM8bit);

		return $DEM8bit;
	}

	public function getDEMAs8BitDEMObject(){
		$DEM8bit=new DEM_Object($this->ncols,$this->nrows,$this->xllcorner,$this->yllcorner,$this->cellsize,$this->nodata_value,$this->getDEMAs8BitArray(),$this->prj);
		return $DEM8bit;
	}

	public function saveToFile($dateiname, $ueberschreiben=false){

		$dateistamm=ShowFileName($dateiname);
		$erweiterung=strtolower(ShowFileExtension($dateiname));

		switch($erweiterung){
			case "png":
				$worldfile_dateiname=$dateistamm.".pgw";
				break;
			case "tif":
			case "tiff":
				$worldfile_dateiname=$dateistamm.".tfw";
				break;
			default:
				$worldfile_dateiname=$dateistamm.".worldfile";
				break;
		}

		$buffer="";
		$buffer.=implode("\n",array(
			"ncols         ".$this->ncols,
			"nrows         ".$this->nrows,
			"xllcorner     ".$this->xllcorner,
			"yllcorner     ".$this->yllcorner,
			"cellsize      ".$this->cellsize,
			"NODATA_value  ".$this->nodata_value
		));

		foreach($this->daten as $zeile){
			$buffer.="\n".implode(" ",$zeile);
		}
		
		if(!$ueberschreiben&&(file_exists($dateiname)||file_exists($worldfile_dateiname)||file_exists($dateistamm.".prj"))){
			echo "\nDatei existiert bereits. Nicht gespeichert.\n";
			return -1;

		}else{
			file_put_contents($dateiname, $buffer);

			$worldfile="";
			$worldfile.=sprintf("%0.12f\n%0.12f\n%0.12f\n%0.12f\n%0.12f\n%0.12f", 
				$this->cellsize,
				0,
				0,
				($this->cellsize*(-1)),
				($this->xllcorner+$this->cellsize/2),
				($this->yllcorner+$this->nrows*$this->cellsize-$this->cellsize/2)
			);
			
			file_put_contents($worldfile_dateiname, $worldfile);

			file_put_contents($dateistamm.".prj", $this->prj);

			echo "\nDateien ${dateiname}, ${worldfile_dateiname} und ${dateistamm}.prj gespeichert.\n";
			return true;
		}
		
	}

	public function saveHypsometricImageToFile($dateiname, $farben, $ueberschreiben){

		/* GD::ImageSetPixel ist FURCHTBAAAAR langsam. 
                   ===========================================

		$bild=ImageCreateTrueColor($this->getNumberOfColumns(),$this->getNumberOfRows());

		for($i=0;$i<count($farben["farbwerte"]);$i++){
			sscanf($farben["farbwerte"][$i], 
				"#%2x%2x%2x",
				$rot,
				$gruen,
				$blau);
			$farben["AllocatedColors"][$i]=ImageColorAllocate($bild,$rot,$gruen,$blau);
			echo "\nsetze farbe $i auf $rot $gruen $blau - input:".$farben["farbwerte"][$i]."\n";
		}

		echo "\nGeneriere Farbstufen-Bild ";
		$anzahl_klassen=count($farben["grenzen"]);
		for($i=0;$i<$this->getNumberOfRows();$i++){
			echo "\nReihe $i: \n";			
			for($k=0;$k<$this->getNumberOfColumns();$k++){
				$wert=$this->getValueAt($i,$k);
				$n=-1;

				while($n<$anzahl_klassen){
					if($wert<=$farben["grenzen"][++$n]){
						ImageSetPixel($bild,$i,$k,$farben["AllocatedColors"][$n]);
						echo "."; //"fortschrittsbalken"
						break;
					}
				}
			}
		}

		if(!$ueberschreiben&&(file_exists($dateiname)||file_exists(basename($dateiname, ".png").".pgw"))){
			echo "\nDatei existiert bereits. Nicht gespeichert.\n";
			return -1;

		}else{
			ImagePng($bild,$dateiname);

			$worldfile="";
			$worldfile.=sprintf("%0.12f\n%0.12f\n%0.12f\n%0.12f\n%0.12f\n%0.12f", 
				$this->cellsize,
				0,
				0,
				($this->cellsize*(-1)),
				($this->xllcorner+$this->cellsize/2),
				($this->yllcorner+$this->nrows*$this->cellsize-$this->cellsize/2)
			);
			
			file_put_contents(basename($dateiname, ".png").".pgw", $worldfile);

			echo "\nDatei $dateiname gespeichert.\n";
			return true;
		}
	*/	



		/* Anderer Zugang: PPM-Format schreiben (=ASCII) und mit ImageMagick umwandeln
		   ===========================================================================
		   PPM-Spezifikationen ua. unter http://netpbm.sourceforge.net/doc/ppm.html */
		
		echo "\nGeneriere Farbstufen-Bild ";

		$dateistamm=ShowFileName($dateiname);
		$erweiterung=strtolower(ShowFileExtension($dateiname));

		switch($erweiterung){
			case "png":
				$worldfile_dateiname=$dateistamm.".pgw";
				break;
			case "tif":
			case "tiff":
				$worldfile_dateiname=$dateistamm.".tfw";
				break;
			default:
				$worldfile_dateiname=$dateistamm.".worldfile";
				break;
		}

		for($i=0;$i<count($farben["farbwerte"]);$i++){
			sscanf($farben["farbwerte"][$i], 
				"#%2x%2x%2x",
				$rot,
				$gruen,
				$blau);
			$farben["PPMZeile"][$i]=sprintf("%3d %3d %3d",$rot, $gruen, $blau);
			if(DEBUG) echo "\nsetze farbe $i auf $rot $gruen $blau - input:".$farben["farbwerte"][$i]."\n";
		}

		$spannweite_und_minimum=array($this->getRange(), $this->getMinimum()); //array_walk erlaubt nur einen zusatzparameter
		array_walk($farben["grenzen"],create_function('&$grenze,$schluessel,$spannweite_und_minimum','$grenze=round($grenze*$spannweite_und_minimum[0])+$spannweite_und_minimum[1];return;'),$spannweite_und_minimum);

		if(DEBUG) print_r($farben);

		$bild=$this->daten;
		$anzahl_klassen=count($farben["grenzen"]);
		array_walk_recursive($bild, create_function('&$wert,$schluessel,&$farben','
			if(is_array($wert)) return;			
			$n=-1;			
			while($n<count($farben["grenzen"])){
				if($wert<=$farben["grenzen"][++$n]){
					$wert=$farben["PPMZeile"][$n];
					break;
				}
			}
		return;
		'),$farben);

		if(DEBUG) print_r($bild);

		$PPMPuffer="P3 ".$this->getNumberOfColumns()." ".$this->getNumberOfRows()." 255\n";
		for($i=0;$i<$this->getNumberOfRows();$i++){
			$PPMPuffer.=implode(" ",$bild[$i])."\n";
		}
		
		$temp_dateiname=tempdir()."tmp.ppm";
		if(DEBUG) echo "\n$temp_dateiname\n";

		file_put_contents($temp_dateiname, $PPMPuffer);

		if(!$ueberschreiben&&(file_exists($dateiname)||file_exists($worldfile_dateiname)||file_exists($dateistamm.".prj"))){
			echo "\nDatei existiert bereits. Nicht gespeichert.\n";
			return -1;

		}else{
			system("convert $temp_dateiname $dateiname");
			unlink($temp_dateiname);

			$worldfile="";
			$worldfile.=sprintf("%0.12f\n%0.12f\n%0.12f\n%0.12f\n%0.12f\n%0.12f", 
				$this->cellsize,
				0,
				0,
				($this->cellsize*(-1)),
				($this->xllcorner+$this->cellsize/2),
				($this->yllcorner+$this->nrows*$this->cellsize-$this->cellsize/2)
			);
			
			file_put_contents($worldfile_dateiname, $worldfile);

			file_put_contents($dateistamm.".prj", $this->prj);

			echo "\nDateien ${dateiname}, ${worldfile_dateiname} und ${dateistamm}.prj gespeichert.\n";
			return true;
		}

		

	}

	function saveHypsometricGoogleEarthOverlayToKMZFile($dateiname, $farben, $ueberschreiben){
		$temp_verzeichnis=tempdir();
		$dateistamm=basename(ShowFileName($dateiname));
		
		$this->saveHypsometricImageToFile(
			"${temp_verzeichnis}/${dateistamm}_.tiff",
			$farben,
			true
		);
		
		system("gdalwarp -r lanczos -co TFW=YES -s_srs ${temp_verzeichnis}/${dateistamm}_.prj -t_srs EPSG:4326 ${temp_verzeichnis}/${dateistamm}_.tiff ${temp_verzeichnis}/${dateistamm}.tif");
		rename(".tfw", "${temp_verzeichnis}/${dateistamm}.tfw"); //.tfw ist im PWD

		// tmp-Dateien aufraeumen
		unlink("${temp_verzeichnis}/${dateistamm}_.tiff");
		unlink("${temp_verzeichnis}/${dateistamm}_.prj");
		unlink("${temp_verzeichnis}/${dateistamm}_.tfw");

		// Boundaries der neuen Datei einlesen
		exec("gdalinfo -noct ${temp_verzeichnis}/${dateistamm}.tif",$rueckgabe);
		if(DEBUG) print_r($rueckgabe);
		
		foreach($rueckgabe as $gdalinfo_zeile){
			if(strpos($gdalinfo_zeile,"Upper Left")!==false){
				sscanf($gdalinfo_zeile,"Upper Left  (%f, %f)", $osten, $norden);
				if(DEBUG) echo "\nN $norden \nO $osten \n";
			}
			if(strpos($gdalinfo_zeile,"Lower Right")!==false){
				sscanf($gdalinfo_zeile,"Lower Right (%f, %f)", $westen, $sueden);
				if(DEBUG) echo "\nS $sueden \nW $westen \n";
			}
			if(strpos($gdalinfo_zeile,"Center")!==false){
				sscanf($gdalinfo_zeile,"Center      (%f, %f)", $lookat_lon, $lookat_lat);
				if(DEBUG) echo "\nlat $lookat_lat \nlon $lookat_lon \n";
			}
		}

		$kml_roh='<?xml version="1.0" encoding="UTF-8"?>
<kml xmlns="http://www.opengis.net/kml/2.2" xmlns:gx="http://www.google.com/kml/ext/2.2">
<GroundOverlay>
	<name>KVAF-ChristophFink</name>
	<Icon>
		<href>{{HREF}}</href>
		<viewBoundScale>0.75</viewBoundScale>
	</Icon>
	<LatLonBox>
		<north>{{NORDEN}}</north>
		<south>{{SUEDEN}}</south>
		<east>{{OSTEN}}</east>
		<west>{{WESTEN}}</west>
	</LatLonBox>
	<LookAt>
		<longitude>{{LOOKAT_LON}}</longitude>
		<latitude>{{LOOKAT_LAT}}</latitude>
		<range>60000</range>
		<tilt>25</tilt>
		<heading>-70</heading>
	</LookAt>
</GroundOverlay>
</kml>';
		file_put_contents(
			"${temp_verzeichnis}/${dateistamm}.kml",
			str_replace(
				array(
					"{{NORDEN}}",
					"{{SUEDEN}}",
					"{{OSTEN}}",
					"{{WESTEN}}",
					"{{HREF}}",
					"{{LOOKAT_LON}}",
					"{{LOOKAT_LAT}}",
				),
				array(
					$norden,
					$sueden,
					$osten,
					$westen,
					$dateistamm.".tif",
					$lookat_lon,
					$lookat_lat
				),
				$kml_roh
			)
		);

		if(!$ueberschreiben&&file_exists($dateiname)){
			echo "\nDatei existiert bereits. Nicht gespeichert.\n";
		}else{
			if(file_exists($dateiname))
				unlink($dateiname);			
			system("zip -j $dateiname ${temp_verzeichnis}/${dateistamm}.tif ${temp_verzeichnis}/${dateistamm}.tfw ${temp_verzeichnis}/${dateistamm}.kml");
			echo "\n Datei ${dateiname} geschrieben.\n";
		}

		// tmp-Dateien aufraeumen
		unlink("${temp_verzeichnis}/${dateistamm}.tfw");
		unlink("${temp_verzeichnis}/${dateistamm}.kml");
		unlink("${temp_verzeichnis}/${dateistamm}.tif");
		rmdir($temp_verzeichnis);
		
	}

		
}



//---------hilfsfunktionen
function array_sum_recursive($array) { //verändert von: http://at2.php.net/manual/en/function.array-sum.php#85375
	$func = create_function('&$arr=0,$key=0','static $a=0; if(is_int($arr)) { $a+=$arr; } return $a;');
	array_walk_recursive($array,$func);
	return $func();
}

function array_flatten($ar) { //von http://stackoverflow.com/questions/526556/how-to-flatten-a-multi-dimensional-array-to-simple-one-in-php
	$toflat = array($ar);
	$res = array();

	while (($r = array_shift($toflat)) !== NULL) {
		foreach ($r as $v) {
			if (is_array($v)) {
					$toflat[] = $v;
			} else {
				$res[] = $v;
			}
		}
	}
	return $res;
}

if (!function_exists('file_put_contents')) { //von http://at.php.net/manual/en/function.file-put-contents.php#86864
    function file_put_contents($filename, $data) {
        $f = @fopen($filename, 'w');
        if (!$f) {
            return false;
        } else {
            $bytes = fwrite($f, $data);
            fclose($f);
            return $bytes;
        }
    }
}

function tempdir($dir=false,$prefix='php') { //von http://at.php.net/manual/de/function.tempnam.php#88544
    $tempfile=tempnam('','');
    if (file_exists($tempfile)) { unlink($tempfile); }
    mkdir($tempfile);
    if (is_dir($tempfile)) { return $tempfile; }
}

function ShowFileExtension($filepath){ //verändert von http://at.php.net/manual/en/function.basename.php#89127
        preg_match('/[^?]*/', $filepath, $matches);
        $string = $matches[0];
     
        $pattern = preg_split('/\./', $string, -1, PREG_SPLIT_OFFSET_CAPTURE);

        # check if there is any extension
        if(count($pattern) == 1)
        {
            if(DEBUG) echo 'No File Extension Present';
            return "";
        }
       
        if(count($pattern) > 1)
        {
            $filenamepart = $pattern[count($pattern)-1][0];
            preg_match('/[^?]*/', $filenamepart, $matches);
            return $matches[0];
        }
}
   
function ShowFileName($filepath){ //verändert von http://at.php.net/manual/en/function.basename.php#89127
        preg_match('/[^?]*/', $filepath, $matches);
        $string = $matches[0];
        #split the string by the literal dot in the filename
        $pattern = preg_split('/\./', $string, -1, PREG_SPLIT_OFFSET_CAPTURE);
        #get the last dot position
        $lastdot = $pattern[count($pattern)-1][1];
        #now extract the filename using the basename function
        $filename = dirname($filepath)."/".basename(substr($string, 0, $lastdot-1));
        #return the filename part
        return $filename;
} 

//-------------------------






//-------------------------//
// H A U P T R O U T I N E //
//-------------------------//

// Interaktives Auswählen deaktiviert 
// ----------------------------------

$read_file="php://stdin";
$handle=fopen($read_file,"rw");

system("ls -l");

echo "Eingabedatei? ";
$eingabe=fgets($handle);
fclose($handle);


$dateistamm=ShowFileName($eingabe);

$dem=new DEM_Object(trim($eingabe));

echo "\n\nDas geladene DEM ".trim($eingabe)." weist folgende statistischen Merkmale auf:\n--\n";

printf("% -24s: % 7d\n", "Maximum", $dem->getMaximum());
printf("% -24s: % 7d\n", "Minimum", $dem->getMinimum());
printf("% -24s: % 7d\n", "Spannweite", $dem->getRange());
printf("% -24s: % 10.2f\n", "Arithmetisches Mittel", $dem->getMean());
printf("% -24s: % 10.2f\n", "Median", $dem->getMedian());
printf("% -24s: % 7d\n", "Anzahl Zeilen", $dem->getNumberOfRows());
printf("% -24s: % 7d\n", "Anzahl Spalten", $dem->getNumberOfColumns());

$dem_klein=$dem->getDEMAs8BitDEMObject();

$dem_klein->saveToFile("${dateistamm}_8bit.asc", true);

$dem_klein->saveHypsometricImageToFile(
	"${dateistamm}_farbstufen.png",
	array(
		"grenzen"=>array(0.2,0.4,0.6,0.8,1),
		"farbwerte"=>array("#c0e470","#fff64c","#f68636","#df6531","#ca5122")
	),
	true
);


// PROGRAMM 2
/*$dem->saveHypsometricGoogleEarthOverlayToKMZFile(
	"${dateistamm}.kmz",
	array(
		"grenzen"=>array(0.2,0.4,0.6,0.8,1),
		"farbwerte"=>array("#c0e470","#fff64c","#f68636","#df6531","#ca5122")
	),
	true
);*/



//-------------------------//


		





?>
